# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ....._base_client import AsyncPaginator, make_request_options
from .....types.zero_trust.casb.posture import export_list_params
from .....types.zero_trust.casb.posture.export_get_response import ExportGetResponse
from .....types.zero_trust.casb.posture.export_list_response import ExportListResponse

__all__ = ["ExportsResource", "AsyncExportsResource"]


class ExportsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ExportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ExportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ExportsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        status: Literal["Pending", "Success", "Failure", "Rescheduled", "In-Progress"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[ExportListResponse]:
        """
        List all export jobs for a given requestor's organization

        Args:
          page: A page number within the paginated result set.

          per_page: Number of results to return per page.

          status: Filter on export job's status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/data-security/posture/exports", account_id=account_id),
            page=SyncV4PagePaginationArray[ExportListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    export_list_params.ExportListParams,
                ),
            ),
            model=ExportListResponse,
        )

    def get(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ExportGetResponse]:
        """
        Retrieves a single export job by its unique identifier

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/accounts/{account_id}/data-security/posture/exports/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ExportGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ExportGetResponse]], ResultWrapper[ExportGetResponse]),
        )


class AsyncExportsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncExportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncExportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncExportsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        status: Literal["Pending", "Success", "Failure", "Rescheduled", "In-Progress"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ExportListResponse, AsyncV4PagePaginationArray[ExportListResponse]]:
        """
        List all export jobs for a given requestor's organization

        Args:
          page: A page number within the paginated result set.

          per_page: Number of results to return per page.

          status: Filter on export job's status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/data-security/posture/exports", account_id=account_id),
            page=AsyncV4PagePaginationArray[ExportListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    export_list_params.ExportListParams,
                ),
            ),
            model=ExportListResponse,
        )

    async def get(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ExportGetResponse]:
        """
        Retrieves a single export job by its unique identifier

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/data-security/posture/exports/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ExportGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ExportGetResponse]], ResultWrapper[ExportGetResponse]),
        )


class ExportsResourceWithRawResponse:
    def __init__(self, exports: ExportsResource) -> None:
        self._exports = exports

        self.list = to_raw_response_wrapper(
            exports.list,
        )
        self.get = to_raw_response_wrapper(
            exports.get,
        )


class AsyncExportsResourceWithRawResponse:
    def __init__(self, exports: AsyncExportsResource) -> None:
        self._exports = exports

        self.list = async_to_raw_response_wrapper(
            exports.list,
        )
        self.get = async_to_raw_response_wrapper(
            exports.get,
        )


class ExportsResourceWithStreamingResponse:
    def __init__(self, exports: ExportsResource) -> None:
        self._exports = exports

        self.list = to_streamed_response_wrapper(
            exports.list,
        )
        self.get = to_streamed_response_wrapper(
            exports.get,
        )


class AsyncExportsResourceWithStreamingResponse:
    def __init__(self, exports: AsyncExportsResource) -> None:
        self._exports = exports

        self.list = async_to_streamed_response_wrapper(
            exports.list,
        )
        self.get = async_to_streamed_response_wrapper(
            exports.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["WebhookEvaluateExistingResponse"]


class WebhookEvaluateExistingResponse(BaseModel):
    """Response body for webhook evaluation test results."""

    message: str
    """Human-readable message describing the test result."""

    status_code: int
    """HTTP status code returned by the webhook endpoint. 0 if connection failed."""

    success: bool
    """Whether the webhook test was successful (received 2xx response)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.custom_csrs import custom_csr_list_params, custom_csr_create_params
from ...types.custom_csrs.custom_csr_get_response import CustomCsrGetResponse
from ...types.custom_csrs.custom_csr_list_response import CustomCsrListResponse
from ...types.custom_csrs.custom_csr_create_response import CustomCsrCreateResponse
from ...types.custom_csrs.custom_csr_delete_response import CustomCsrDeleteResponse

__all__ = ["CustomCsrsResource", "AsyncCustomCsrsResource"]


class CustomCsrsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomCsrsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CustomCsrsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomCsrsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CustomCsrsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        common_name: str,
        country: str,
        locality: str,
        organization: str,
        sans: SequenceNotStr[str],
        state: str,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        description: str | Omit = omit,
        key_type: Literal["rsa2048", "p256v1"] | Omit = omit,
        name: str | Omit = omit,
        organizational_unit: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCsrCreateResponse]:
        """
        Generate a new custom Certificate Signing Request (CSR) for an account or zone.
        Cloudflare generates and securely stores the private key associated with the
        CSR.

        Args:
          common_name: The common name (domain) for the CSR. Must be at most 64 characters.

          country: Two-letter ISO 3166-1 alpha-2 country code.

          locality: City or locality name.

          organization: Organization name.

          sans: Subject Alternative Names for the CSR. At least one SAN is required.

          state: State or province name.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          description: Optional description for the CSR.

          key_type: Key algorithm to use for the CSR. Defaults to rsa2048 if not specified.

          name: Human-readable name for the CSR.

          organizational_unit: Organizational unit name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_csrs",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=maybe_transform(
                {
                    "common_name": common_name,
                    "country": country,
                    "locality": locality,
                    "organization": organization,
                    "sans": sans,
                    "state": state,
                    "description": description,
                    "key_type": key_type,
                    "name": name,
                    "organizational_unit": organizational_unit,
                },
                custom_csr_create_params.CustomCsrCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCsrCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomCsrCreateResponse]], ResultWrapper[CustomCsrCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[CustomCsrListResponse]:
        """
        List all custom Certificate Signing Requests (CSRs) for an account or zone.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          page: Page number of paginated results.

          per_page: Number of custom CSRs per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_csrs",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncV4PagePaginationArray[CustomCsrListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    custom_csr_list_params.CustomCsrListParams,
                ),
            ),
            model=CustomCsrListResponse,
        )

    def delete(
        self,
        custom_csr_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCsrDeleteResponse]:
        """
        Delete a custom Certificate Signing Request (CSR) and its associated private
        key.

        Args:
          custom_csr_id: Custom CSR identifier tag.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not custom_csr_id:
            raise ValueError(f"Expected a non-empty value for `custom_csr_id` but received {custom_csr_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._delete(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_csrs/{custom_csr_id}",
                custom_csr_id=custom_csr_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCsrDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomCsrDeleteResponse]], ResultWrapper[CustomCsrDeleteResponse]),
        )

    def get(
        self,
        custom_csr_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCsrGetResponse]:
        """
        Retrieve details for a specific custom Certificate Signing Request (CSR).

        Args:
          custom_csr_id: Custom CSR identifier tag.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not custom_csr_id:
            raise ValueError(f"Expected a non-empty value for `custom_csr_id` but received {custom_csr_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_csrs/{custom_csr_id}",
                custom_csr_id=custom_csr_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCsrGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomCsrGetResponse]], ResultWrapper[CustomCsrGetResponse]),
        )


class AsyncCustomCsrsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomCsrsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomCsrsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomCsrsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCustomCsrsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        common_name: str,
        country: str,
        locality: str,
        organization: str,
        sans: SequenceNotStr[str],
        state: str,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        description: str | Omit = omit,
        key_type: Literal["rsa2048", "p256v1"] | Omit = omit,
        name: str | Omit = omit,
        organizational_unit: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCsrCreateResponse]:
        """
        Generate a new custom Certificate Signing Request (CSR) for an account or zone.
        Cloudflare generates and securely stores the private key associated with the
        CSR.

        Args:
          common_name: The common name (domain) for the CSR. Must be at most 64 characters.

          country: Two-letter ISO 3166-1 alpha-2 country code.

          locality: City or locality name.

          organization: Organization name.

          sans: Subject Alternative Names for the CSR. At least one SAN is required.

          state: State or province name.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          description: Optional description for the CSR.

          key_type: Key algorithm to use for the CSR. Defaults to rsa2048 if not specified.

          name: Human-readable name for the CSR.

          organizational_unit: Organizational unit name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_csrs",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=await async_maybe_transform(
                {
                    "common_name": common_name,
                    "country": country,
                    "locality": locality,
                    "organization": organization,
                    "sans": sans,
                    "state": state,
                    "description": description,
                    "key_type": key_type,
                    "name": name,
                    "organizational_unit": organizational_unit,
                },
                custom_csr_create_params.CustomCsrCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCsrCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomCsrCreateResponse]], ResultWrapper[CustomCsrCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomCsrListResponse, AsyncV4PagePaginationArray[CustomCsrListResponse]]:
        """
        List all custom Certificate Signing Requests (CSRs) for an account or zone.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          page: Page number of paginated results.

          per_page: Number of custom CSRs per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_csrs",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncV4PagePaginationArray[CustomCsrListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    custom_csr_list_params.CustomCsrListParams,
                ),
            ),
            model=CustomCsrListResponse,
        )

    async def delete(
        self,
        custom_csr_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCsrDeleteResponse]:
        """
        Delete a custom Certificate Signing Request (CSR) and its associated private
        key.

        Args:
          custom_csr_id: Custom CSR identifier tag.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not custom_csr_id:
            raise ValueError(f"Expected a non-empty value for `custom_csr_id` but received {custom_csr_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._delete(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_csrs/{custom_csr_id}",
                custom_csr_id=custom_csr_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCsrDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomCsrDeleteResponse]], ResultWrapper[CustomCsrDeleteResponse]),
        )

    async def get(
        self,
        custom_csr_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomCsrGetResponse]:
        """
        Retrieve details for a specific custom Certificate Signing Request (CSR).

        Args:
          custom_csr_id: Custom CSR identifier tag.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not custom_csr_id:
            raise ValueError(f"Expected a non-empty value for `custom_csr_id` but received {custom_csr_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._get(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_csrs/{custom_csr_id}",
                custom_csr_id=custom_csr_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomCsrGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomCsrGetResponse]], ResultWrapper[CustomCsrGetResponse]),
        )


class CustomCsrsResourceWithRawResponse:
    def __init__(self, custom_csrs: CustomCsrsResource) -> None:
        self._custom_csrs = custom_csrs

        self.create = to_raw_response_wrapper(
            custom_csrs.create,
        )
        self.list = to_raw_response_wrapper(
            custom_csrs.list,
        )
        self.delete = to_raw_response_wrapper(
            custom_csrs.delete,
        )
        self.get = to_raw_response_wrapper(
            custom_csrs.get,
        )


class AsyncCustomCsrsResourceWithRawResponse:
    def __init__(self, custom_csrs: AsyncCustomCsrsResource) -> None:
        self._custom_csrs = custom_csrs

        self.create = async_to_raw_response_wrapper(
            custom_csrs.create,
        )
        self.list = async_to_raw_response_wrapper(
            custom_csrs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            custom_csrs.delete,
        )
        self.get = async_to_raw_response_wrapper(
            custom_csrs.get,
        )


class CustomCsrsResourceWithStreamingResponse:
    def __init__(self, custom_csrs: CustomCsrsResource) -> None:
        self._custom_csrs = custom_csrs

        self.create = to_streamed_response_wrapper(
            custom_csrs.create,
        )
        self.list = to_streamed_response_wrapper(
            custom_csrs.list,
        )
        self.delete = to_streamed_response_wrapper(
            custom_csrs.delete,
        )
        self.get = to_streamed_response_wrapper(
            custom_csrs.get,
        )


class AsyncCustomCsrsResourceWithStreamingResponse:
    def __init__(self, custom_csrs: AsyncCustomCsrsResource) -> None:
        self._custom_csrs = custom_csrs

        self.create = async_to_streamed_response_wrapper(
            custom_csrs.create,
        )
        self.list = async_to_streamed_response_wrapper(
            custom_csrs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            custom_csrs.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            custom_csrs.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Union, Iterable, Optional, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from .instances import (
    InstancesResource,
    AsyncInstancesResource,
    InstancesResourceWithRawResponse,
    AsyncInstancesResourceWithRawResponse,
    InstancesResourceWithStreamingResponse,
    AsyncInstancesResourceWithStreamingResponse,
)
from ......_types import (
    Body,
    Omit,
    Query,
    Headers,
    NotGiven,
    SequenceNotStr,
    Base64FileInput,
    omit,
    not_given,
)
from ......_utils import path_template, maybe_transform, async_maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_wrappers import ResultWrapper
from ......pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ......_base_client import AsyncPaginator, make_request_options
from ......types.zero_trust.casb.posture import (
    finding_list_params,
    finding_export_params,
    finding_ignore_params,
    finding_unignore_params,
    finding_tune_severity_params,
)
from ......types.zero_trust.casb.posture.finding_get_response import FindingGetResponse
from ......types.zero_trust.casb.posture.finding_list_response import FindingListResponse
from ......types.zero_trust.casb.posture.finding_export_response import FindingExportResponse
from ......types.zero_trust.casb.posture.finding_ignore_response import FindingIgnoreResponse
from ......types.zero_trust.casb.posture.finding_unignore_response import FindingUnignoreResponse
from ......types.zero_trust.casb.posture.finding_tune_severity_response import FindingTuneSeverityResponse
from ......types.zero_trust.casb.posture.finding_reset_severity_response import FindingResetSeverityResponse

__all__ = ["FindingsResource", "AsyncFindingsResource"]


class FindingsResource(SyncAPIResource):
    @cached_property
    def instances(self) -> InstancesResource:
        return InstancesResource(self._client)

    @cached_property
    def with_raw_response(self) -> FindingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FindingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FindingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FindingsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        finding_type_ids: str | Omit = omit,
        ignored: bool | Omit = omit,
        integration_id: str | Omit = omit,
        max_affliction_date: Union[str, datetime] | Omit = omit,
        min_affliction_date: Union[str, datetime] | Omit = omit,
        observation: Literal["Activity", "Insight", "Issue"] | Omit = omit,
        order: Literal["finding.name", "instance_count", "integration.name", "latest_affliction_date", "severity"]
        | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        product: Literal["Cloud", "Saas"] | Omit = omit,
        search: str | Omit = omit,
        severity: Literal["Critical", "High", "Medium", "Low"] | Omit = omit,
        type: Literal["Content", "Posture"] | Omit = omit,
        vendor: Literal[
            "ANTHROPIC",
            "AWS",
            "BITBUCKET",
            "BOX",
            "CONFLUENCE",
            "DROPBOX",
            "GITHUB",
            "GOOGLE_CLOUD_PLATFORM",
            "GOOGLE_WORKSPACE",
            "JIRA",
            "MICROSOFT",
            "MICROSOFT_INTERNAL",
            "OPENAI",
            "SALESFORCE",
            "SERVICENOW",
            "SLACK",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[FindingListResponse]:
        """List all security findings that have been identified as being problematic.

        This
        will return a list of findings regardless if they have been ignored or not.

        Args:
          cursor: A cursor for pagination. Obtained from the `result_info.cursor` field of a
              previous response.

          direction: Direction to order results.

          finding_type_ids: A comma separated list of UUIDs identifying the finding type(s).

          ignored: Filter for only the ignored findings. Set to false to only see "active" items

          integration_id: Filter by an integration ID

          max_affliction_date: Filter to view findings that occurred on or before the affliction date. Can be a
              date-time in ISO 8601 format or an epoch timestamp.

          min_affliction_date: Filter to view findings that occurred on or after the affliction date. Can be a
              date-time in ISO 8601 format or an epoch timestamp.

          observation: Filter by observation type of the finding

          order: Which field to use when ordering the findings.

          page: A page number within the paginated result set.

          per_page: Number of results to return per page.

          product: Filter by product category of the finding

          search: A search term.

          severity: Filter by severity

          type: Filter by type of the finding

          vendor: Filter by vendor

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/data-security/posture/findings", account_id=account_id),
            page=SyncV4PagePaginationArray[FindingListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "direction": direction,
                        "finding_type_ids": finding_type_ids,
                        "ignored": ignored,
                        "integration_id": integration_id,
                        "max_affliction_date": max_affliction_date,
                        "min_affliction_date": min_affliction_date,
                        "observation": observation,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "product": product,
                        "search": search,
                        "severity": severity,
                        "type": type,
                        "vendor": vendor,
                    },
                    finding_list_params.FindingListParams,
                ),
            ),
            model=FindingListResponse,
        )

    def export(
        self,
        *,
        account_id: str,
        ignored: bool | Omit = omit,
        integration_id: SequenceNotStr[str] | Omit = omit,
        max_affliction_date: Union[str, datetime] | Omit = omit,
        min_affliction_date: Union[str, datetime] | Omit = omit,
        orders: Iterable[finding_export_params.Order] | Omit = omit,
        product: Optional[Literal["SaaS", "Cloud"]] | Omit = omit,
        search: str | Omit = omit,
        severities: List[Literal["CRITICAL", "HIGH", "MEDIUM", "LOW"]] | Omit = omit,
        vendors: List[
            Literal[
                "ANTHROPIC",
                "AWS",
                "BITBUCKET",
                "BOX",
                "CONFLUENCE",
                "DROPBOX",
                "GITHUB",
                "GOOGLE_CLOUD_PLATFORM",
                "GOOGLE_WORKSPACE",
                "JIRA",
                "MICROSOFT",
                "MICROSOFT_INTERNAL",
                "OPENAI",
                "SALESFORCE",
                "SERVICENOW",
                "SLACK",
            ]
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingExportResponse]:
        """
        Creates a CSV export for findings and accepts optional filters in the payload.

        Args:
          ignored: Filter for only the ignored findings. Set to false to only see active items.

          integration_id: Filter by multiple integration IDs.

          max_affliction_date: Filter to view findings that occurred on or before the affliction date. Can be a
              date-time in ISO 8601 format or an epoch timestamp.

          min_affliction_date: Filter to view findings that occurred on or after the affliction date. Can be a
              date-time in ISO 8601 format or an epoch timestamp.

          orders: Which fields to use when ordering the findings.

          product: Filter by finding's category product.

          search: A search term.

          severities: Filter by severity levels.

          vendors: Filter by vendor types.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/data-security/posture/findings/export", account_id=account_id),
            body=maybe_transform(
                {
                    "ignored": ignored,
                    "integration_id": integration_id,
                    "max_affliction_date": max_affliction_date,
                    "min_affliction_date": min_affliction_date,
                    "orders": orders,
                    "product": product,
                    "search": search,
                    "severities": severities,
                    "vendors": vendors,
                },
                finding_export_params.FindingExportParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingExportResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingExportResponse]], ResultWrapper[FindingExportResponse]),
        )

    def get(
        self,
        finding_id: Union[str, Base64FileInput],
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingGetResponse]:
        """
        Gets a security Finding that has been identified as being problematic.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not finding_id:
            raise ValueError(f"Expected a non-empty value for `finding_id` but received {finding_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/data-security/posture/findings/{finding_id}",
                account_id=account_id,
                finding_id=finding_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingGetResponse]], ResultWrapper[FindingGetResponse]),
        )

    def ignore(
        self,
        *,
        account_id: str,
        checks: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingIgnoreResponse]:
        """Given a list of findings, mark as ignored.

        Does nothing if Finding is already
        ignored.

        Args:
          checks: A list of finding IDs to pass along.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/data-security/posture/findings/ignore", account_id=account_id),
            body=maybe_transform({"checks": checks}, finding_ignore_params.FindingIgnoreParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingIgnoreResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingIgnoreResponse]], ResultWrapper[FindingIgnoreResponse]),
        )

    def reset_severity(
        self,
        finding_id: Union[str, Base64FileInput],
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingResetSeverityResponse]:
        """If a Finding's severity has been changed, reset it back to default value.

        Does
        nothing if no override exists.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not finding_id:
            raise ValueError(f"Expected a non-empty value for `finding_id` but received {finding_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/data-security/posture/findings/{finding_id}/reset_finding_severity",
                account_id=account_id,
                finding_id=finding_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingResetSeverityResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingResetSeverityResponse]], ResultWrapper[FindingResetSeverityResponse]),
        )

    def tune_severity(
        self,
        finding_id: Union[str, Base64FileInput],
        *,
        account_id: str,
        new_severity: Literal[1, 2, 3, 4],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingTuneSeverityResponse]:
        """Update the severity of a Finding.

        This will update the `severity_override` field
        on the Finding payload with the new severity value.

        Args:
          new_severity: The numeric severity value to apply to the finding.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not finding_id:
            raise ValueError(f"Expected a non-empty value for `finding_id` but received {finding_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/data-security/posture/findings/{finding_id}/tune_finding_severity",
                account_id=account_id,
                finding_id=finding_id,
            ),
            body=maybe_transform(
                {"new_severity": new_severity}, finding_tune_severity_params.FindingTuneSeverityParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingTuneSeverityResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingTuneSeverityResponse]], ResultWrapper[FindingTuneSeverityResponse]),
        )

    def unignore(
        self,
        *,
        account_id: str,
        checks: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingUnignoreResponse]:
        """Ability to un-ignore a Finding if it's previously been ignored.

        Does nothing if
        the Finding is not ignored.

        Args:
          checks: A list of finding IDs to pass along.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/data-security/posture/findings/unignore", account_id=account_id),
            body=maybe_transform({"checks": checks}, finding_unignore_params.FindingUnignoreParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingUnignoreResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingUnignoreResponse]], ResultWrapper[FindingUnignoreResponse]),
        )


class AsyncFindingsResource(AsyncAPIResource):
    @cached_property
    def instances(self) -> AsyncInstancesResource:
        return AsyncInstancesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFindingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFindingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFindingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFindingsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        finding_type_ids: str | Omit = omit,
        ignored: bool | Omit = omit,
        integration_id: str | Omit = omit,
        max_affliction_date: Union[str, datetime] | Omit = omit,
        min_affliction_date: Union[str, datetime] | Omit = omit,
        observation: Literal["Activity", "Insight", "Issue"] | Omit = omit,
        order: Literal["finding.name", "instance_count", "integration.name", "latest_affliction_date", "severity"]
        | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        product: Literal["Cloud", "Saas"] | Omit = omit,
        search: str | Omit = omit,
        severity: Literal["Critical", "High", "Medium", "Low"] | Omit = omit,
        type: Literal["Content", "Posture"] | Omit = omit,
        vendor: Literal[
            "ANTHROPIC",
            "AWS",
            "BITBUCKET",
            "BOX",
            "CONFLUENCE",
            "DROPBOX",
            "GITHUB",
            "GOOGLE_CLOUD_PLATFORM",
            "GOOGLE_WORKSPACE",
            "JIRA",
            "MICROSOFT",
            "MICROSOFT_INTERNAL",
            "OPENAI",
            "SALESFORCE",
            "SERVICENOW",
            "SLACK",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[FindingListResponse, AsyncV4PagePaginationArray[FindingListResponse]]:
        """List all security findings that have been identified as being problematic.

        This
        will return a list of findings regardless if they have been ignored or not.

        Args:
          cursor: A cursor for pagination. Obtained from the `result_info.cursor` field of a
              previous response.

          direction: Direction to order results.

          finding_type_ids: A comma separated list of UUIDs identifying the finding type(s).

          ignored: Filter for only the ignored findings. Set to false to only see "active" items

          integration_id: Filter by an integration ID

          max_affliction_date: Filter to view findings that occurred on or before the affliction date. Can be a
              date-time in ISO 8601 format or an epoch timestamp.

          min_affliction_date: Filter to view findings that occurred on or after the affliction date. Can be a
              date-time in ISO 8601 format or an epoch timestamp.

          observation: Filter by observation type of the finding

          order: Which field to use when ordering the findings.

          page: A page number within the paginated result set.

          per_page: Number of results to return per page.

          product: Filter by product category of the finding

          search: A search term.

          severity: Filter by severity

          type: Filter by type of the finding

          vendor: Filter by vendor

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/data-security/posture/findings", account_id=account_id),
            page=AsyncV4PagePaginationArray[FindingListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "direction": direction,
                        "finding_type_ids": finding_type_ids,
                        "ignored": ignored,
                        "integration_id": integration_id,
                        "max_affliction_date": max_affliction_date,
                        "min_affliction_date": min_affliction_date,
                        "observation": observation,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "product": product,
                        "search": search,
                        "severity": severity,
                        "type": type,
                        "vendor": vendor,
                    },
                    finding_list_params.FindingListParams,
                ),
            ),
            model=FindingListResponse,
        )

    async def export(
        self,
        *,
        account_id: str,
        ignored: bool | Omit = omit,
        integration_id: SequenceNotStr[str] | Omit = omit,
        max_affliction_date: Union[str, datetime] | Omit = omit,
        min_affliction_date: Union[str, datetime] | Omit = omit,
        orders: Iterable[finding_export_params.Order] | Omit = omit,
        product: Optional[Literal["SaaS", "Cloud"]] | Omit = omit,
        search: str | Omit = omit,
        severities: List[Literal["CRITICAL", "HIGH", "MEDIUM", "LOW"]] | Omit = omit,
        vendors: List[
            Literal[
                "ANTHROPIC",
                "AWS",
                "BITBUCKET",
                "BOX",
                "CONFLUENCE",
                "DROPBOX",
                "GITHUB",
                "GOOGLE_CLOUD_PLATFORM",
                "GOOGLE_WORKSPACE",
                "JIRA",
                "MICROSOFT",
                "MICROSOFT_INTERNAL",
                "OPENAI",
                "SALESFORCE",
                "SERVICENOW",
                "SLACK",
            ]
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingExportResponse]:
        """
        Creates a CSV export for findings and accepts optional filters in the payload.

        Args:
          ignored: Filter for only the ignored findings. Set to false to only see active items.

          integration_id: Filter by multiple integration IDs.

          max_affliction_date: Filter to view findings that occurred on or before the affliction date. Can be a
              date-time in ISO 8601 format or an epoch timestamp.

          min_affliction_date: Filter to view findings that occurred on or after the affliction date. Can be a
              date-time in ISO 8601 format or an epoch timestamp.

          orders: Which fields to use when ordering the findings.

          product: Filter by finding's category product.

          search: A search term.

          severities: Filter by severity levels.

          vendors: Filter by vendor types.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/data-security/posture/findings/export", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "ignored": ignored,
                    "integration_id": integration_id,
                    "max_affliction_date": max_affliction_date,
                    "min_affliction_date": min_affliction_date,
                    "orders": orders,
                    "product": product,
                    "search": search,
                    "severities": severities,
                    "vendors": vendors,
                },
                finding_export_params.FindingExportParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingExportResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingExportResponse]], ResultWrapper[FindingExportResponse]),
        )

    async def get(
        self,
        finding_id: Union[str, Base64FileInput],
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingGetResponse]:
        """
        Gets a security Finding that has been identified as being problematic.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not finding_id:
            raise ValueError(f"Expected a non-empty value for `finding_id` but received {finding_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/data-security/posture/findings/{finding_id}",
                account_id=account_id,
                finding_id=finding_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingGetResponse]], ResultWrapper[FindingGetResponse]),
        )

    async def ignore(
        self,
        *,
        account_id: str,
        checks: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingIgnoreResponse]:
        """Given a list of findings, mark as ignored.

        Does nothing if Finding is already
        ignored.

        Args:
          checks: A list of finding IDs to pass along.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/data-security/posture/findings/ignore", account_id=account_id),
            body=await async_maybe_transform({"checks": checks}, finding_ignore_params.FindingIgnoreParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingIgnoreResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingIgnoreResponse]], ResultWrapper[FindingIgnoreResponse]),
        )

    async def reset_severity(
        self,
        finding_id: Union[str, Base64FileInput],
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingResetSeverityResponse]:
        """If a Finding's severity has been changed, reset it back to default value.

        Does
        nothing if no override exists.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not finding_id:
            raise ValueError(f"Expected a non-empty value for `finding_id` but received {finding_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/data-security/posture/findings/{finding_id}/reset_finding_severity",
                account_id=account_id,
                finding_id=finding_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingResetSeverityResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingResetSeverityResponse]], ResultWrapper[FindingResetSeverityResponse]),
        )

    async def tune_severity(
        self,
        finding_id: Union[str, Base64FileInput],
        *,
        account_id: str,
        new_severity: Literal[1, 2, 3, 4],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingTuneSeverityResponse]:
        """Update the severity of a Finding.

        This will update the `severity_override` field
        on the Finding payload with the new severity value.

        Args:
          new_severity: The numeric severity value to apply to the finding.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not finding_id:
            raise ValueError(f"Expected a non-empty value for `finding_id` but received {finding_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/data-security/posture/findings/{finding_id}/tune_finding_severity",
                account_id=account_id,
                finding_id=finding_id,
            ),
            body=await async_maybe_transform(
                {"new_severity": new_severity}, finding_tune_severity_params.FindingTuneSeverityParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingTuneSeverityResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingTuneSeverityResponse]], ResultWrapper[FindingTuneSeverityResponse]),
        )

    async def unignore(
        self,
        *,
        account_id: str,
        checks: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FindingUnignoreResponse]:
        """Ability to un-ignore a Finding if it's previously been ignored.

        Does nothing if
        the Finding is not ignored.

        Args:
          checks: A list of finding IDs to pass along.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/data-security/posture/findings/unignore", account_id=account_id),
            body=await async_maybe_transform({"checks": checks}, finding_unignore_params.FindingUnignoreParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FindingUnignoreResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FindingUnignoreResponse]], ResultWrapper[FindingUnignoreResponse]),
        )


class FindingsResourceWithRawResponse:
    def __init__(self, findings: FindingsResource) -> None:
        self._findings = findings

        self.list = to_raw_response_wrapper(
            findings.list,
        )
        self.export = to_raw_response_wrapper(
            findings.export,
        )
        self.get = to_raw_response_wrapper(
            findings.get,
        )
        self.ignore = to_raw_response_wrapper(
            findings.ignore,
        )
        self.reset_severity = to_raw_response_wrapper(
            findings.reset_severity,
        )
        self.tune_severity = to_raw_response_wrapper(
            findings.tune_severity,
        )
        self.unignore = to_raw_response_wrapper(
            findings.unignore,
        )

    @cached_property
    def instances(self) -> InstancesResourceWithRawResponse:
        return InstancesResourceWithRawResponse(self._findings.instances)


class AsyncFindingsResourceWithRawResponse:
    def __init__(self, findings: AsyncFindingsResource) -> None:
        self._findings = findings

        self.list = async_to_raw_response_wrapper(
            findings.list,
        )
        self.export = async_to_raw_response_wrapper(
            findings.export,
        )
        self.get = async_to_raw_response_wrapper(
            findings.get,
        )
        self.ignore = async_to_raw_response_wrapper(
            findings.ignore,
        )
        self.reset_severity = async_to_raw_response_wrapper(
            findings.reset_severity,
        )
        self.tune_severity = async_to_raw_response_wrapper(
            findings.tune_severity,
        )
        self.unignore = async_to_raw_response_wrapper(
            findings.unignore,
        )

    @cached_property
    def instances(self) -> AsyncInstancesResourceWithRawResponse:
        return AsyncInstancesResourceWithRawResponse(self._findings.instances)


class FindingsResourceWithStreamingResponse:
    def __init__(self, findings: FindingsResource) -> None:
        self._findings = findings

        self.list = to_streamed_response_wrapper(
            findings.list,
        )
        self.export = to_streamed_response_wrapper(
            findings.export,
        )
        self.get = to_streamed_response_wrapper(
            findings.get,
        )
        self.ignore = to_streamed_response_wrapper(
            findings.ignore,
        )
        self.reset_severity = to_streamed_response_wrapper(
            findings.reset_severity,
        )
        self.tune_severity = to_streamed_response_wrapper(
            findings.tune_severity,
        )
        self.unignore = to_streamed_response_wrapper(
            findings.unignore,
        )

    @cached_property
    def instances(self) -> InstancesResourceWithStreamingResponse:
        return InstancesResourceWithStreamingResponse(self._findings.instances)


class AsyncFindingsResourceWithStreamingResponse:
    def __init__(self, findings: AsyncFindingsResource) -> None:
        self._findings = findings

        self.list = async_to_streamed_response_wrapper(
            findings.list,
        )
        self.export = async_to_streamed_response_wrapper(
            findings.export,
        )
        self.get = async_to_streamed_response_wrapper(
            findings.get,
        )
        self.ignore = async_to_streamed_response_wrapper(
            findings.ignore,
        )
        self.reset_severity = async_to_streamed_response_wrapper(
            findings.reset_severity,
        )
        self.tune_severity = async_to_streamed_response_wrapper(
            findings.tune_severity,
        )
        self.unignore = async_to_streamed_response_wrapper(
            findings.unignore,
        )

    @cached_property
    def instances(self) -> AsyncInstancesResourceWithStreamingResponse:
        return AsyncInstancesResourceWithStreamingResponse(self._findings.instances)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from ....._utils import PropertyInfo
from ....._models import BaseModel
from ....workers.scripts.consumer_script import ConsumerScript

__all__ = [
    "ScriptUpdateResponse",
    "CacheOptions",
    "Exports",
    "ExportsWorkersWorkerExport",
    "ExportsWorkersWorkerExportCache",
    "ExportsWorkersDurableObjectExport",
    "ExportsWorkersDurableObjectDeletedExport",
    "ExportsWorkersDurableObjectRenamedExport",
    "ExportsWorkersDurableObjectTransferredExport",
    "ExportsWorkersDurableObjectExpectingTransferExport",
    "NamedHandler",
    "Observability",
    "ObservabilityLogs",
    "ObservabilityTraces",
    "Placement",
    "PlacementUnionMember0",
    "PlacementUnionMember1",
    "PlacementUnionMember2",
    "PlacementUnionMember3",
    "PlacementUnionMember4",
    "PlacementUnionMember5",
    "PlacementUnionMember6",
    "PlacementUnionMember7",
    "PlacementUnionMember7Target",
    "PlacementUnionMember7TargetRegion",
    "PlacementUnionMember7TargetHostname",
    "PlacementUnionMember7TargetHost",
]


class CacheOptions(BaseModel):
    """Global CacheW configuration for the Worker.

    When caching is on,
    the platform provisions a `cloudflare.app` zone for the Worker.
    A `type: worker` entry in the `exports` map can override this
    value for a single entrypoint.
    """

    enabled: bool
    """Whether caching is enabled for this Worker."""

    cross_version_cache: Optional[bool] = None
    """Whether cached responses are shared across Worker version uploads.

    This is independent of `enabled`. It can stay true while caching is off, so the
    preference survives turning caching off and back on.
    """


class ExportsWorkersWorkerExportCache(BaseModel):
    """Cache override for this entrypoint.

    Overrides the Worker's
    global `cache_options.enabled` for this entrypoint only.
    """

    enabled: bool
    """Whether caching is enabled for this entrypoint."""


class ExportsWorkersWorkerExport(BaseModel):
    """A named Worker entrypoint export (`type: worker`).

    Worker
    entrypoints are always live (`state: created`) and carry no
    storage or lifecycle fields. The optional `cache` block overrides
    the Worker's global `cache_options.enabled` for this entrypoint.
    """

    type: Literal["worker"]
    """Marks this entry as a Worker entrypoint export."""

    cache: Optional[ExportsWorkersWorkerExportCache] = None
    """Cache override for this entrypoint.

    Overrides the Worker's global `cache_options.enabled` for this entrypoint only.
    """

    state: Optional[Literal["created"]] = None
    """Live export. May be omitted; defaults to `created`."""


class ExportsWorkersDurableObjectExport(BaseModel):
    """A live Durable Object export (`state: created`, the default).

    The
    platform auto-provisions the namespace on first deploy, matches it
    on subsequent deploys, and never mutates or deletes it as a side
    effect of a code-only change. `storage` is required; `renamed_to`,
    `transferred_to` and `transfer_from` are not allowed on a live
    entry.
    """

    storage: Literal["sqlite", "legacy-kv"]
    """Durable Object storage backend.

    `sqlite` is the recommended (and only) backend for new namespaces. `legacy-kv`
    is accepted only for a class whose namespace already exists as KV-backed; the
    `exports` flow never provisions a new `legacy-kv` namespace.
    """

    type: Literal["durable-object"]
    """Marks this entry as a Durable Object export."""

    container: Optional[str] = None
    """
    Name of the container (declared in the upload's `metadata.containers`) that
    backs this Durable Object. When set, the namespace is container-enabled. Valid
    only on live entries.
    """

    state: Optional[Literal["created"]] = None
    """Live export. May be omitted; defaults to `created`."""


class ExportsWorkersDurableObjectDeletedExport(BaseModel):
    """
    A `deleted` tombstone: retires the provisioned namespace for this
    class and all of its data. The class must be absent from the
    uploaded code and no other Worker in the account may bind to the
    namespace, otherwise the deploy is rejected. No other fields are
    allowed. Deletion is irreversible.
    """

    state: Literal["deleted"]
    """Tombstone that deletes the namespace."""

    type: Literal["durable-object"]
    """Marks this entry as a Durable Object export."""


class ExportsWorkersDurableObjectRenamedExport(BaseModel):
    """
    A `renamed` tombstone: rewrites the provisioned namespace's class
    name from this map key to `renamed_to`. The source class may stay
    in code during the rollout window (an info notice is emitted).
    `storage`, `transferred_to` and `transfer_from` are not allowed.
    """

    state: Literal["renamed"]
    """Tombstone that renames the namespace's class."""

    type: Literal["durable-object"]
    """Marks this entry as a Durable Object export."""


class ExportsWorkersDurableObjectTransferredExport(BaseModel):
    """
    A `transferred` tombstone (source side of a two-phase transfer):
    hands ownership of the provisioned namespace to another script in
    the same account, named by `transferred_to`. The target must have
    already deployed a matching `expecting-transfer` entry. The source
    class may stay in code during the rollout window (an info notice
    is emitted). `storage`, `renamed_to` and `transfer_from` are not
    allowed.
    """

    state: Literal["transferred"]
    """Tombstone that transfers the namespace to another script."""

    type: Literal["durable-object"]
    """Marks this entry as a Durable Object export."""


class ExportsWorkersDurableObjectExpectingTransferExport(BaseModel):
    """The target side of a two-phase transfer (`state:
    expecting-transfer`).

    Declares that this script expects to receive
    a namespace for this class from the `transfer_from` script. This
    is a live entry, not a tombstone: bindings resolve through the
    source's namespace until the source commits with a `transferred`
    tombstone. `storage` and `transfer_from` are required; `renamed_to`
    and `transferred_to` are not allowed.
    """

    state: Literal["expecting-transfer"]
    """Target side of a two-phase transfer."""

    storage: Literal["sqlite", "legacy-kv"]
    """Durable Object storage backend.

    `sqlite` is the recommended (and only) backend for new namespaces. `legacy-kv`
    is accepted only for a class whose namespace already exists as KV-backed; the
    `exports` flow never provisions a new `legacy-kv` namespace.
    """

    transfer_from: str
    """The source script name to receive the namespace from.

    Must be in the same account and dispatch-namespace context. Present on reads for
    `expecting-transfer` entries.
    """

    type: Literal["durable-object"]
    """Marks this entry as a Durable Object export."""

    container: Optional[str] = None
    """
    Name of the container (declared in the upload's `metadata.containers`) that
    backs this Durable Object once the transfer settles. Valid only on live entries.
    """


Exports: TypeAlias = Annotated[
    Union[
        ExportsWorkersWorkerExport,
        ExportsWorkersDurableObjectExport,
        ExportsWorkersDurableObjectDeletedExport,
        ExportsWorkersDurableObjectRenamedExport,
        ExportsWorkersDurableObjectTransferredExport,
        ExportsWorkersDurableObjectExpectingTransferExport,
    ],
    PropertyInfo(discriminator="type"),
]


class NamedHandler(BaseModel):
    handlers: Optional[List[str]] = None
    """The names of handlers exported as part of the named export."""

    name: Optional[str] = None
    """The name of the export."""


class ObservabilityLogs(BaseModel):
    """Log settings for the Worker."""

    enabled: bool
    """Whether logs are enabled for the Worker."""

    invocation_logs: bool
    """
    Whether
    [invocation logs](https://developers.cloudflare.com/workers/observability/logs/workers-logs/#invocation-logs)
    are enabled for the Worker.
    """

    destinations: Optional[List[str]] = None
    """A list of destinations where logs will be exported to."""

    head_sampling_rate: Optional[float] = None
    """The sampling rate for logs. From 0 to 1 (1 = 100%, 0.1 = 10%). Default is 1."""

    persist: Optional[bool] = None
    """Whether log persistence is enabled for the Worker."""


class ObservabilityTraces(BaseModel):
    """Trace settings for the Worker."""

    destinations: Optional[List[str]] = None
    """A list of destinations where traces will be exported to."""

    enabled: Optional[bool] = None
    """Whether traces are enabled for the Worker."""

    head_sampling_rate: Optional[float] = None
    """The sampling rate for traces. From 0 to 1 (1 = 100%, 0.1 = 10%). Default is 1."""

    persist: Optional[bool] = None
    """Whether trace persistence is enabled for the Worker."""

    propagation_policy: Optional[Literal["authenticated", "accept"]] = None
    """
    Controls how inbound trace context (traceparent/tracestate) headers on incoming
    requests are handled. "authenticated" (default) honors inbound trace context
    only when accompanied by a valid trace auth token. "accept" unconditionally
    accepts inbound trace context. Requires the trace propagation feature to be
    enabled.
    """


class Observability(BaseModel):
    """Observability settings for the Worker."""

    enabled: bool
    """Whether observability is enabled for the Worker."""

    head_sampling_rate: Optional[float] = None
    """The sampling rate for incoming requests.

    From 0 to 1 (1 = 100%, 0.1 = 10%). Default is 1.
    """

    logs: Optional[ObservabilityLogs] = None
    """Log settings for the Worker."""

    traces: Optional[ObservabilityTraces] = None
    """Trace settings for the Worker."""


class PlacementUnionMember0(BaseModel):
    mode: Literal["smart"]
    """
    Enables
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """

    last_analyzed_at: Optional[datetime] = None
    """
    The last time the script was analyzed for
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """

    status: Optional[Literal["SUCCESS", "UNSUPPORTED_APPLICATION", "INSUFFICIENT_INVOCATIONS"]] = None
    """
    Status of
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """


class PlacementUnionMember1(BaseModel):
    region: str
    """Cloud region for targeted placement in format 'provider:region'."""

    last_analyzed_at: Optional[datetime] = None
    """
    The last time the script was analyzed for
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """

    status: Optional[Literal["SUCCESS", "UNSUPPORTED_APPLICATION", "INSUFFICIENT_INVOCATIONS"]] = None
    """
    Status of
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """


class PlacementUnionMember2(BaseModel):
    hostname: str
    """HTTP hostname for targeted placement."""

    last_analyzed_at: Optional[datetime] = None
    """
    The last time the script was analyzed for
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """

    status: Optional[Literal["SUCCESS", "UNSUPPORTED_APPLICATION", "INSUFFICIENT_INVOCATIONS"]] = None
    """
    Status of
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """


class PlacementUnionMember3(BaseModel):
    host: str
    """TCP host and port for targeted placement."""

    last_analyzed_at: Optional[datetime] = None
    """
    The last time the script was analyzed for
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """

    status: Optional[Literal["SUCCESS", "UNSUPPORTED_APPLICATION", "INSUFFICIENT_INVOCATIONS"]] = None
    """
    Status of
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """


class PlacementUnionMember4(BaseModel):
    mode: Literal["targeted"]
    """Targeted placement mode."""

    region: str
    """Cloud region for targeted placement in format 'provider:region'."""

    last_analyzed_at: Optional[datetime] = None
    """
    The last time the script was analyzed for
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """

    status: Optional[Literal["SUCCESS", "UNSUPPORTED_APPLICATION", "INSUFFICIENT_INVOCATIONS"]] = None
    """
    Status of
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """


class PlacementUnionMember5(BaseModel):
    hostname: str
    """HTTP hostname for targeted placement."""

    mode: Literal["targeted"]
    """Targeted placement mode."""

    last_analyzed_at: Optional[datetime] = None
    """
    The last time the script was analyzed for
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """

    status: Optional[Literal["SUCCESS", "UNSUPPORTED_APPLICATION", "INSUFFICIENT_INVOCATIONS"]] = None
    """
    Status of
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """


class PlacementUnionMember6(BaseModel):
    host: str
    """TCP host and port for targeted placement."""

    mode: Literal["targeted"]
    """Targeted placement mode."""

    last_analyzed_at: Optional[datetime] = None
    """
    The last time the script was analyzed for
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """

    status: Optional[Literal["SUCCESS", "UNSUPPORTED_APPLICATION", "INSUFFICIENT_INVOCATIONS"]] = None
    """
    Status of
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """


class PlacementUnionMember7TargetRegion(BaseModel):
    region: str
    """Cloud region in format 'provider:region'."""


class PlacementUnionMember7TargetHostname(BaseModel):
    hostname: str
    """HTTP hostname for targeted placement."""


class PlacementUnionMember7TargetHost(BaseModel):
    host: str
    """TCP host:port for targeted placement."""


PlacementUnionMember7Target: TypeAlias = Union[
    PlacementUnionMember7TargetRegion, PlacementUnionMember7TargetHostname, PlacementUnionMember7TargetHost
]


class PlacementUnionMember7(BaseModel):
    mode: Literal["targeted"]
    """Targeted placement mode."""

    target: List[PlacementUnionMember7Target]
    """Array of placement targets (currently limited to single target)."""

    last_analyzed_at: Optional[datetime] = None
    """
    The last time the script was analyzed for
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """

    status: Optional[Literal["SUCCESS", "UNSUPPORTED_APPLICATION", "INSUFFICIENT_INVOCATIONS"]] = None
    """
    Status of
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """


Placement: TypeAlias = Union[
    PlacementUnionMember0,
    PlacementUnionMember1,
    PlacementUnionMember2,
    PlacementUnionMember3,
    PlacementUnionMember4,
    PlacementUnionMember5,
    PlacementUnionMember6,
    PlacementUnionMember7,
]


class ScriptUpdateResponse(BaseModel):
    startup_time_ms: int

    id: Optional[str] = None
    """The name used to identify the script."""

    cache_options: Optional[CacheOptions] = None
    """Global CacheW configuration for the Worker.

    When caching is on, the platform provisions a `cloudflare.app` zone for the
    Worker. A `type: worker` entry in the `exports` map can override this value for
    a single entrypoint.
    """

    compatibility_date: Optional[str] = None
    """Date indicating targeted support in the Workers runtime.

    Backwards incompatible fixes to the runtime following this date will not affect
    this Worker.
    """

    compatibility_flags: Optional[List[str]] = None
    """Flags that enable or disable certain features in the Workers runtime.

    Used to enable upcoming features or opt in or out of specific changes not
    included in a `compatibility_date`.
    """

    created_on: Optional[datetime] = None
    """When the script was created."""

    entry_point: Optional[str] = None
    """The entry point for the script."""

    etag: Optional[str] = None
    """Hashed script content, can be used in a If-None-Match header when updating."""

    exports: Optional[Dict[str, Exports]] = None
    """
    Declarative exports for the Worker's most recent version, including Durable
    Object classes (with their `storage` backend) and named Worker entrypoints.
    Tombstoned lifecycle entries are omitted, so only live exports (`created` and
    `expecting-transfer`) are returned.
    """

    handlers: Optional[List[str]] = None
    """The names of handlers exported as part of the default export."""

    has_assets: Optional[bool] = None
    """Whether a Worker contains assets."""

    has_modules: Optional[bool] = None
    """Whether a Worker contains modules."""

    last_deployed_from: Optional[str] = None
    """The client most recently used to deploy this Worker."""

    logpush: Optional[bool] = None
    """Whether Logpush is turned on for the Worker."""

    migration_tag: Optional[str] = None
    """
    The tag of the Durable Object migration that was most recently applied for this
    Worker.
    """

    modified_on: Optional[datetime] = None
    """When the script was last modified."""

    named_handlers: Optional[List[NamedHandler]] = None
    """
    Named exports, such as Durable Object class implementations and named
    entrypoints.
    """

    observability: Optional[Observability] = None
    """Observability settings for the Worker."""

    placement: Optional[Placement] = None
    """
    Configuration for
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    Specify mode='smart' for Smart Placement, or one of region/hostname/host.
    """

    placement_mode: Optional[Literal["smart", "targeted"]] = None

    placement_status: Optional[Literal["SUCCESS", "UNSUPPORTED_APPLICATION", "INSUFFICIENT_INVOCATIONS"]] = None

    tag: Optional[str] = None
    """The immutable ID of the script."""

    tags: Optional[List[str]] = None
    """Tags associated with the Worker."""

    tail_consumers: Optional[List[ConsumerScript]] = None
    """List of Workers that will consume logs from the attached Worker."""

    usage_model: Optional[Literal["standard", "bundled", "unbound"]] = None
    """Usage model for the Worker invocations."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["InterruptCreateParams", "Reboot", "Restart", "Shutdown"]


class InterruptCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier"""

    reboot: Reboot

    restart: Restart

    shutdown: Shutdown


class Reboot(TypedDict, total=False):
    purge: bool
    """Purge connector state."""


class Restart(TypedDict, total=False):
    purge: bool
    """Purge connector state."""


class Shutdown(TypedDict, total=False):
    purge: bool
    """Purge connector state."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .registration_edit_params import RegistrationEditParams as RegistrationEditParams
from .registration_list_params import RegistrationListParams as RegistrationListParams
from .registration_get_response import RegistrationGetResponse as RegistrationGetResponse
from .registration_create_params import RegistrationCreateParams as RegistrationCreateParams
from .registration_edit_response import RegistrationEditResponse as RegistrationEditResponse
from .registration_list_response import RegistrationListResponse as RegistrationListResponse
from .update_status_get_response import UpdateStatusGetResponse as UpdateStatusGetResponse
from .registration_create_response import RegistrationCreateResponse as RegistrationCreateResponse
from .registrar_sandbox_check_params import RegistrarSandboxCheckParams as RegistrarSandboxCheckParams
from .registrar_sandbox_search_params import RegistrarSandboxSearchParams as RegistrarSandboxSearchParams
from .registrar_sandbox_check_response import RegistrarSandboxCheckResponse as RegistrarSandboxCheckResponse
from .registration_status_get_response import RegistrationStatusGetResponse as RegistrationStatusGetResponse
from .registrar_sandbox_search_response import RegistrarSandboxSearchResponse as RegistrarSandboxSearchResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SuperBotFightModeDefinitelyConfiguration", "StaleZoneConfiguration"]


class StaleZoneConfiguration(BaseModel):
    """
    A read-only field that shows which unauthorized settings are currently active on the zone. These settings typically result from upgrades or downgrades.
    """

    fight_mode: Optional[bool] = None
    """Indicates that the zone's Bot Fight Mode is turned on."""

    sbfm_likely_automated: Optional[str] = None
    """
    Indicates that the zone's likely automated requests are being blocked or
    challenged.
    """


class SuperBotFightModeDefinitelyConfiguration(BaseModel):
    ai_bots_protection: Optional[Literal["block", "disabled", "only_on_ad_pages"]] = None
    """Enable rule to block AI Scrapers and Crawlers."""

    cf_robots_variant: Optional[Literal["off", "policy_only"]] = None
    """Specifies the Robots Access Control License variant to use."""

    content_bots_protection: Optional[Literal["block", "disabled"]] = None
    """Enable rule to block content bots.

    When enabled, blocks automated traffic with low bot scores, excluding safe
    verified bot categories. Exceptions should be managed via skip rules.
    """

    crawler_protection: Optional[Literal["enabled", "disabled"]] = None
    """Enable rule to punish AI Scrapers and Crawlers via a link maze."""

    enable_js: Optional[bool] = None
    """Use lightweight, invisible JavaScript detections to improve Bot Management.

    [Learn more about JavaScript Detections](https://developers.cloudflare.com/bots/reference/javascript-detections/).
    """

    is_robots_txt_managed: Optional[bool] = None
    """Enable cloudflare managed robots.txt.

    If an existing robots.txt is detected, then managed robots.txt will be prepended
    to the existing robots.txt.
    """

    optimize_wordpress: Optional[bool] = None
    """Whether to optimize Super Bot Fight Mode protections for Wordpress."""

    sbfm_definitely_automated: Optional[Literal["allow", "block", "managed_challenge"]] = None
    """Super Bot Fight Mode (SBFM) action to take on definitely automated requests."""

    sbfm_static_resource_protection: Optional[bool] = None
    """
    Super Bot Fight Mode (SBFM) to enable static resource protection. Enable if
    static resources on your application need bot protection. Note: Static resource
    protection can also result in legitimate traffic being blocked.
    """

    sbfm_verified_bots: Optional[Literal["allow", "block"]] = None
    """Super Bot Fight Mode (SBFM) action to take on verified bots requests."""

    stale_zone_configuration: Optional[StaleZoneConfiguration] = None
    """
    A read-only field that shows which unauthorized settings are currently active on
    the zone. These settings typically result from upgrades or downgrades.
    """

    using_latest_model: Optional[bool] = None
    """
    A read-only field that indicates whether the zone currently is running the
    latest ML model.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["InstanceStepResponse", "Error"]


class Error(BaseModel):
    """Error details when status='errored'; null otherwise."""

    message: str

    name: str


class InstanceStepResponse(BaseModel):
    error: Optional[Error] = None
    """Error details when status='errored'; null otherwise."""

    status: Literal[
        "queued", "running", "paused", "errored", "terminated", "complete", "waitingForPause", "waiting", "rollingBack"
    ]

    output: Optional[object] = None
    """Full step output or waitForEvent payload without truncation.

    Sensitive outputs are returned as '[REDACTED]'. Populated when
    status='complete'. May be a ReadableStream when the step returned one from
    step.do; stream outputs are served as application/octet-stream rather than JSON.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import TypeAlias

from ...._models import BaseModel

__all__ = [
    "QueryGetResponse",
    "UnionMember0",
    "UnionMember0Parameters",
    "UnionMember0ParametersStringMatch",
    "UnionMember1",
    "UnionMember1Parameters",
    "UnionMember1ParametersStringMatch",
]


class UnionMember0ParametersStringMatch(BaseModel):
    pattern: str


class UnionMember0Parameters(BaseModel):
    string_matches: List[UnionMember0ParametersStringMatch]

    max_time: Optional[str] = None

    min_time: Optional[str] = None


class UnionMember0(BaseModel):
    created: str

    parameters: Optional[UnionMember0Parameters] = None

    query_id: int

    query_tag: str

    scan: bool

    updated: str


class UnionMember1ParametersStringMatch(BaseModel):
    pattern: str


class UnionMember1Parameters(BaseModel):
    string_matches: List[UnionMember1ParametersStringMatch]

    max_time: Optional[str] = None

    min_time: Optional[str] = None


class UnionMember1(BaseModel):
    created: str

    parameters: Optional[UnionMember1Parameters] = None

    query_id: int

    query_tag: str

    scan: bool

    updated: str


QueryGetResponse: TypeAlias = Union[List[UnionMember0], UnionMember1]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["InstanceStepParams"]


class InstanceStepParams(TypedDict, total=False):
    account_id: Required[str]

    workflow_name: Required[str]

    name: Required[str]
    """
    Exact step name from the instance logs response, including the generated counter
    suffix.
    """

    type: Required[Literal["step", "waitForEvent"]]
    """
    Step type to disambiguate step.do and waitForEvent entries that share the same
    name.
    """

    attempt: int
    """Specific attempt number to retrieve output or error for."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Union, Optional, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.moq.relays import token_create_params
from ....types.moq.relays.token_list_response import TokenListResponse
from ....types.moq.relays.token_create_response import TokenCreateResponse
from ....types.moq.relays.token_delete_response import TokenDeleteResponse

__all__ = ["TokensResource", "AsyncTokensResource"]


class TokensResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TokensResourceWithStreamingResponse(self)

    def create(
        self,
        relay_id: str,
        *,
        account_id: str,
        operations: List[Literal["publish", "subscribe"]],
        expires: Union[str, datetime] | Omit = omit,
        label: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenCreateResponse]:
        """Mints a new relay-scoped token and adds it to the relay's accepted-auth
        registry.

        The token value (secret) is shown once in the response. A relay may
        hold up to 10 tokens; creating an 11th is rejected.

        Args:
          account_id: Cloudflare account identifier.

          operations: Non-empty subset of the V1 roles the token is allowed to perform. Signed into
              the token.

          expires: Optional expiry (RFC 3339). Defaults to 1 year from creation; rejected if more
              than 1 year in the future.

          label: Optional, customer-set label.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/moq/relays/{relay_id}/tokens", account_id=account_id, relay_id=relay_id
            ),
            body=maybe_transform(
                {
                    "operations": operations,
                    "expires": expires,
                    "label": label,
                },
                token_create_params.TokenCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenCreateResponse]], ResultWrapper[TokenCreateResponse]),
        )

    def list(
        self,
        relay_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenListResponse]:
        """Returns metadata for every token in the relay's registry.

        Secrets are never
        returned. The dashboard derives an `expired` flag by comparing each token's
        `expires` to the current time.

        Args:
          account_id: Cloudflare account identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/moq/relays/{relay_id}/tokens", account_id=account_id, relay_id=relay_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenListResponse]], ResultWrapper[TokenListResponse]),
        )

    def delete(
        self,
        jti: str,
        *,
        account_id: str,
        relay_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TokenDeleteResponse:
        """Revokes a token by removing it from the relay's registry.

        crique rejects the
        token within the cache TTL. Idempotent — revoking an unknown token succeeds.

        Args:
          account_id: Cloudflare account identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        if not jti:
            raise ValueError(f"Expected a non-empty value for `jti` but received {jti!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/moq/relays/{relay_id}/tokens/{jti}",
                account_id=account_id,
                relay_id=relay_id,
                jti=jti,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TokenDeleteResponse,
        )


class AsyncTokensResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTokensResourceWithStreamingResponse(self)

    async def create(
        self,
        relay_id: str,
        *,
        account_id: str,
        operations: List[Literal["publish", "subscribe"]],
        expires: Union[str, datetime] | Omit = omit,
        label: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenCreateResponse]:
        """Mints a new relay-scoped token and adds it to the relay's accepted-auth
        registry.

        The token value (secret) is shown once in the response. A relay may
        hold up to 10 tokens; creating an 11th is rejected.

        Args:
          account_id: Cloudflare account identifier.

          operations: Non-empty subset of the V1 roles the token is allowed to perform. Signed into
              the token.

          expires: Optional expiry (RFC 3339). Defaults to 1 year from creation; rejected if more
              than 1 year in the future.

          label: Optional, customer-set label.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/moq/relays/{relay_id}/tokens", account_id=account_id, relay_id=relay_id
            ),
            body=await async_maybe_transform(
                {
                    "operations": operations,
                    "expires": expires,
                    "label": label,
                },
                token_create_params.TokenCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenCreateResponse]], ResultWrapper[TokenCreateResponse]),
        )

    async def list(
        self,
        relay_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenListResponse]:
        """Returns metadata for every token in the relay's registry.

        Secrets are never
        returned. The dashboard derives an `expired` flag by comparing each token's
        `expires` to the current time.

        Args:
          account_id: Cloudflare account identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/moq/relays/{relay_id}/tokens", account_id=account_id, relay_id=relay_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenListResponse]], ResultWrapper[TokenListResponse]),
        )

    async def delete(
        self,
        jti: str,
        *,
        account_id: str,
        relay_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TokenDeleteResponse:
        """Revokes a token by removing it from the relay's registry.

        crique rejects the
        token within the cache TTL. Idempotent — revoking an unknown token succeeds.

        Args:
          account_id: Cloudflare account identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not relay_id:
            raise ValueError(f"Expected a non-empty value for `relay_id` but received {relay_id!r}")
        if not jti:
            raise ValueError(f"Expected a non-empty value for `jti` but received {jti!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/moq/relays/{relay_id}/tokens/{jti}",
                account_id=account_id,
                relay_id=relay_id,
                jti=jti,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TokenDeleteResponse,
        )


class TokensResourceWithRawResponse:
    def __init__(self, tokens: TokensResource) -> None:
        self._tokens = tokens

        self.create = to_raw_response_wrapper(
            tokens.create,
        )
        self.list = to_raw_response_wrapper(
            tokens.list,
        )
        self.delete = to_raw_response_wrapper(
            tokens.delete,
        )


class AsyncTokensResourceWithRawResponse:
    def __init__(self, tokens: AsyncTokensResource) -> None:
        self._tokens = tokens

        self.create = async_to_raw_response_wrapper(
            tokens.create,
        )
        self.list = async_to_raw_response_wrapper(
            tokens.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tokens.delete,
        )


class TokensResourceWithStreamingResponse:
    def __init__(self, tokens: TokensResource) -> None:
        self._tokens = tokens

        self.create = to_streamed_response_wrapper(
            tokens.create,
        )
        self.list = to_streamed_response_wrapper(
            tokens.list,
        )
        self.delete = to_streamed_response_wrapper(
            tokens.delete,
        )


class AsyncTokensResourceWithStreamingResponse:
    def __init__(self, tokens: AsyncTokensResource) -> None:
        self._tokens = tokens

        self.create = async_to_streamed_response_wrapper(
            tokens.create,
        )
        self.list = async_to_streamed_response_wrapper(
            tokens.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tokens.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel
from .device_input import DeviceInput
from .device_match import DeviceMatch

__all__ = ["DevicePostureRule"]


class DevicePostureRule(BaseModel):
    id: Optional[str] = None
    """API UUID."""

    description: Optional[str] = None
    """The description of the device posture rule."""

    enabled: Optional[bool] = None
    """Whether the rule is enabled.

    This is a computed, read-only value. It is false for deprecated Kolide posture
    rules that still use the issue_count input, and true otherwise.
    """

    expiration: Optional[str] = None
    """Sets the expiration time for a posture check result.

    If empty, the result remains valid until it is overwritten by new data from the
    WARP client.
    """

    input: Optional[DeviceInput] = None
    """The value to be checked against."""

    match: Optional[List[DeviceMatch]] = None
    """The conditions that the client must match to run the rule."""

    name: Optional[str] = None
    """The name of the device posture rule."""

    schedule: Optional[str] = None
    """Polling frequency for the WARP client posture check.

    Default: `5m` (poll every five minutes). Minimum: `1m`.
    """

    type: Optional[
        Literal[
            "file",
            "application",
            "tanium",
            "gateway",
            "warp",
            "disk_encryption",
            "serial_number",
            "sentinelone",
            "carbonblack",
            "firewall",
            "os_version",
            "domain_joined",
            "client_certificate",
            "client_certificate_v2",
            "antivirus",
            "unique_client_id",
            "kolide",
            "tanium_s2s",
            "crowdstrike_s2s",
            "intune",
            "workspace_one",
            "sentinelone_s2s",
            "custom_s2s",
        ]
    ] = None
    """The type of device posture rule."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = [
    "DeviceListResponse",
    "CPUPctByApp",
    "DeviceIPV4",
    "DeviceIPV4Location",
    "DeviceIPV6",
    "DeviceIPV6Location",
    "GatewayIPV4",
    "GatewayIPV4Location",
    "GatewayIPV6",
    "GatewayIPV6Location",
    "ISPIPV4",
    "ISPIPV4Location",
    "ISPIPV6",
    "ISPIPV6Location",
    "RamUsedPctByApp",
    "RTT",
    "RTTMinRTTUs",
    "RTTRTTUs",
    "RTTRTTVarUs",
    "TunnelStats",
    "TunnelStatsBytesLost",
    "TunnelStatsBytesReceived",
    "TunnelStatsBytesRetransmitted",
    "TunnelStatsBytesSent",
    "TunnelStatsPacketsLost",
    "TunnelStatsPacketsReceived",
    "TunnelStatsPacketsRetransmitted",
    "TunnelStatsPacketsSent",
]


class CPUPctByApp(BaseModel):
    cpu_pct: Optional[float] = None
    """CPU usage percentage, on a scale of 0 to 100."""

    name: Optional[str] = None
    """Application name."""


class DeviceIPV4Location(BaseModel):
    city: Optional[str] = None

    country_iso: Optional[str] = None

    state_iso: Optional[str] = None

    zip: Optional[str] = None


class DeviceIPV4(BaseModel):
    address: Optional[str] = None

    asn: Optional[int] = None

    aso: Optional[str] = None

    location: Optional[DeviceIPV4Location] = None

    name: Optional[str] = None

    netmask: Optional[str] = None

    version: Optional[int] = None
    """IP version (`1` for IPv4, `2` for IPv6, `0` if unknown)."""


class DeviceIPV6Location(BaseModel):
    city: Optional[str] = None

    country_iso: Optional[str] = None

    state_iso: Optional[str] = None

    zip: Optional[str] = None


class DeviceIPV6(BaseModel):
    address: Optional[str] = None

    asn: Optional[int] = None

    aso: Optional[str] = None

    location: Optional[DeviceIPV6Location] = None

    name: Optional[str] = None

    netmask: Optional[str] = None

    version: Optional[int] = None
    """IP version (`1` for IPv4, `2` for IPv6, `0` if unknown)."""


class GatewayIPV4Location(BaseModel):
    city: Optional[str] = None

    country_iso: Optional[str] = None

    state_iso: Optional[str] = None

    zip: Optional[str] = None


class GatewayIPV4(BaseModel):
    address: Optional[str] = None

    asn: Optional[int] = None

    aso: Optional[str] = None

    location: Optional[GatewayIPV4Location] = None

    name: Optional[str] = None

    netmask: Optional[str] = None

    version: Optional[int] = None
    """IP version (`1` for IPv4, `2` for IPv6, `0` if unknown)."""


class GatewayIPV6Location(BaseModel):
    city: Optional[str] = None

    country_iso: Optional[str] = None

    state_iso: Optional[str] = None

    zip: Optional[str] = None


class GatewayIPV6(BaseModel):
    address: Optional[str] = None

    asn: Optional[int] = None

    aso: Optional[str] = None

    location: Optional[GatewayIPV6Location] = None

    name: Optional[str] = None

    netmask: Optional[str] = None

    version: Optional[int] = None
    """IP version (`1` for IPv4, `2` for IPv6, `0` if unknown)."""


class ISPIPV4Location(BaseModel):
    city: Optional[str] = None

    country_iso: Optional[str] = None

    state_iso: Optional[str] = None

    zip: Optional[str] = None


class ISPIPV4(BaseModel):
    address: Optional[str] = None

    asn: Optional[int] = None

    aso: Optional[str] = None

    location: Optional[ISPIPV4Location] = None

    name: Optional[str] = None

    netmask: Optional[str] = None

    version: Optional[int] = None
    """IP version (`1` for IPv4, `2` for IPv6, `0` if unknown)."""


class ISPIPV6Location(BaseModel):
    city: Optional[str] = None

    country_iso: Optional[str] = None

    state_iso: Optional[str] = None

    zip: Optional[str] = None


class ISPIPV6(BaseModel):
    address: Optional[str] = None

    asn: Optional[int] = None

    aso: Optional[str] = None

    location: Optional[ISPIPV6Location] = None

    name: Optional[str] = None

    netmask: Optional[str] = None

    version: Optional[int] = None
    """IP version (`1` for IPv4, `2` for IPv6, `0` if unknown)."""


class RamUsedPctByApp(BaseModel):
    name: Optional[str] = None
    """Application name."""

    ram_used_pct: Optional[float] = None
    """RAM usage percentage, on a scale of 0 to 100."""


class RTTMinRTTUs(BaseModel):
    """Minimum round-trip time in microseconds."""

    downstream: Optional[int] = None

    upstream: Optional[int] = None


class RTTRTTUs(BaseModel):
    """Round-trip time in microseconds."""

    downstream: Optional[int] = None

    upstream: Optional[int] = None


class RTTRTTVarUs(BaseModel):
    """Round-trip time variance in microseconds."""

    downstream: Optional[int] = None

    upstream: Optional[int] = None


class RTT(BaseModel):
    """Round-trip time statistics for the WARP tunnel."""

    min_rtt_us: Optional[RTTMinRTTUs] = FieldInfo(alias="minRttUs", default=None)
    """Minimum round-trip time in microseconds."""

    rtt_us: Optional[RTTRTTUs] = FieldInfo(alias="rttUs", default=None)
    """Round-trip time in microseconds."""

    rtt_var_us: Optional[RTTRTTVarUs] = FieldInfo(alias="rttVarUs", default=None)
    """Round-trip time variance in microseconds."""


class TunnelStatsBytesLost(BaseModel):
    """Number of bytes lost, split by direction."""

    downstream: Optional[int] = None

    upstream: Optional[int] = None


class TunnelStatsBytesReceived(BaseModel):
    """Number of bytes received, split by direction."""

    downstream: Optional[int] = None

    upstream: Optional[int] = None


class TunnelStatsBytesRetransmitted(BaseModel):
    """Number of bytes retransmitted, split by direction."""

    downstream: Optional[int] = None

    upstream: Optional[int] = None


class TunnelStatsBytesSent(BaseModel):
    """Number of bytes sent, split by direction."""

    downstream: Optional[int] = None

    upstream: Optional[int] = None


class TunnelStatsPacketsLost(BaseModel):
    """Number of packets lost, split by direction."""

    downstream: Optional[int] = None

    upstream: Optional[int] = None


class TunnelStatsPacketsReceived(BaseModel):
    """Number of packets received, split by direction."""

    downstream: Optional[int] = None

    upstream: Optional[int] = None


class TunnelStatsPacketsRetransmitted(BaseModel):
    """Number of packets retransmitted, split by direction."""

    downstream: Optional[int] = None

    upstream: Optional[int] = None


class TunnelStatsPacketsSent(BaseModel):
    """Number of packets sent, split by direction."""

    downstream: Optional[int] = None

    upstream: Optional[int] = None


class TunnelStats(BaseModel):
    """WARP tunnel packet and byte counters."""

    bytes_lost: Optional[TunnelStatsBytesLost] = FieldInfo(alias="bytesLost", default=None)
    """Number of bytes lost, split by direction."""

    bytes_received: Optional[TunnelStatsBytesReceived] = FieldInfo(alias="bytesReceived", default=None)
    """Number of bytes received, split by direction."""

    bytes_retransmitted: Optional[TunnelStatsBytesRetransmitted] = FieldInfo(alias="bytesRetransmitted", default=None)
    """Number of bytes retransmitted, split by direction."""

    bytes_sent: Optional[TunnelStatsBytesSent] = FieldInfo(alias="bytesSent", default=None)
    """Number of bytes sent, split by direction."""

    packets_lost: Optional[TunnelStatsPacketsLost] = FieldInfo(alias="packetsLost", default=None)
    """Number of packets lost, split by direction."""

    packets_received: Optional[TunnelStatsPacketsReceived] = FieldInfo(alias="packetsReceived", default=None)
    """Number of packets received, split by direction."""

    packets_retransmitted: Optional[TunnelStatsPacketsRetransmitted] = FieldInfo(
        alias="packetsRetransmitted", default=None
    )
    """Number of packets retransmitted, split by direction."""

    packets_sent: Optional[TunnelStatsPacketsSent] = FieldInfo(alias="packetsSent", default=None)
    """Number of packets sent, split by direction."""

    stats_window_ms: Optional[int] = FieldInfo(alias="statsWindowMs", default=None)
    """The measurement window duration in milliseconds."""


class DeviceListResponse(BaseModel):
    colo: str
    """Cloudflare colo airport code."""

    device_id: str = FieldInfo(alias="deviceId")
    """Device identifier (UUID v4)"""

    mode: str
    """The mode under which the WARP client is run."""

    platform: str
    """Operating system."""

    status: str
    """Network status."""

    timestamp: str

    version: str
    """WARP client version."""

    always_on: Optional[bool] = FieldInfo(alias="alwaysOn", default=None)

    battery_charging: Optional[bool] = FieldInfo(alias="batteryCharging", default=None)

    battery_cycles: Optional[int] = FieldInfo(alias="batteryCycles", default=None)

    battery_pct: Optional[float] = FieldInfo(alias="batteryPct", default=None)

    connection_type: Optional[str] = FieldInfo(alias="connectionType", default=None)

    cpu_pct: Optional[float] = FieldInfo(alias="cpuPct", default=None)

    cpu_pct_by_app: Optional[List[CPUPctByApp]] = FieldInfo(alias="cpuPctByApp", default=None)

    device_ipv4: Optional[DeviceIPV4] = FieldInfo(alias="deviceIpv4", default=None)

    device_ipv6: Optional[DeviceIPV6] = FieldInfo(alias="deviceIpv6", default=None)

    device_name: Optional[str] = FieldInfo(alias="deviceName", default=None)
    """Device identifier (human readable)."""

    device_registration: Optional[str] = FieldInfo(alias="deviceRegistration", default=None)
    """Deprecated: use registrationId. Device registration identifier (UUID)."""

    disk_read_bps: Optional[int] = FieldInfo(alias="diskReadBps", default=None)

    disk_usage_pct: Optional[float] = FieldInfo(alias="diskUsagePct", default=None)

    disk_write_bps: Optional[int] = FieldInfo(alias="diskWriteBps", default=None)

    doh_subdomain: Optional[str] = FieldInfo(alias="dohSubdomain", default=None)

    estimated_loss_pct: Optional[float] = FieldInfo(alias="estimatedLossPct", default=None)

    firewall_enabled: Optional[bool] = FieldInfo(alias="firewallEnabled", default=None)

    gateway_ipv4: Optional[GatewayIPV4] = FieldInfo(alias="gatewayIpv4", default=None)

    gateway_ipv6: Optional[GatewayIPV6] = FieldInfo(alias="gatewayIpv6", default=None)

    handshake_latency_ms: Optional[float] = FieldInfo(alias="handshakeLatencyMs", default=None)

    isp_ipv4: Optional[ISPIPV4] = FieldInfo(alias="ispIpv4", default=None)

    isp_ipv6: Optional[ISPIPV6] = FieldInfo(alias="ispIpv6", default=None)

    metal: Optional[str] = None

    network_rcvd_bps: Optional[int] = FieldInfo(alias="networkRcvdBps", default=None)

    network_sent_bps: Optional[int] = FieldInfo(alias="networkSentBps", default=None)

    network_ssid: Optional[str] = FieldInfo(alias="networkSsid", default=None)

    person_email: Optional[str] = FieldInfo(alias="personEmail", default=None)
    """User contact email address"""

    ram_available_kb: Optional[int] = FieldInfo(alias="ramAvailableKb", default=None)

    ram_used_pct: Optional[float] = FieldInfo(alias="ramUsedPct", default=None)

    ram_used_pct_by_app: Optional[List[RamUsedPctByApp]] = FieldInfo(alias="ramUsedPctByApp", default=None)

    registration_id: Optional[str] = FieldInfo(alias="registrationId", default=None)
    """Device registration identifier (UUID v4).

    On multi-user devices, this uniquely identifies a user's registration on the
    device.
    """

    rtt: Optional[RTT] = None
    """Round-trip time statistics for the WARP tunnel."""

    switch_locked: Optional[bool] = FieldInfo(alias="switchLocked", default=None)

    tunnel_stats: Optional[TunnelStats] = FieldInfo(alias="tunnelStats", default=None)
    """WARP tunnel packet and byte counters."""

    tunnel_type: Optional[str] = FieldInfo(alias="tunnelType", default=None)

    wifi_strength_dbm: Optional[int] = FieldInfo(alias="wifiStrengthDbm", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .catalog_list_response import CatalogListResponse as CatalogListResponse
from .by_dataset_list_response import ByDatasetListResponse as ByDatasetListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ...._models import BaseModel

__all__ = [
    "FlagListResponse",
    "Rule",
    "RuleCondition",
    "RuleConditionUnionMember0",
    "RuleConditionUnionMember1",
    "RuleConditionUnionMember1Clause",
    "RuleConditionUnionMember1ClauseUnionMember0",
    "RuleConditionUnionMember1ClauseUnionMember1",
    "RuleConditionUnionMember1ClauseUnionMember1Clause",
    "RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember0",
    "RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1",
    "RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1Clause",
    "RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember0",
    "RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1",
    "RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1Clause",
    "RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember0",
    "RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1",
    "RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1Clause",
    "RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember0",
    "RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1",
    "RuleRollout",
]


class RuleConditionUnionMember0(BaseModel):
    attribute: str

    operator: Literal[
        "equals",
        "not_equals",
        "greater_than",
        "less_than",
        "greater_than_or_equals",
        "less_than_or_equals",
        "contains",
        "starts_with",
        "ends_with",
        "in",
        "not_in",
    ]

    value: Union[Optional[str], float, bool, Dict[str, object], List[object]]


class RuleConditionUnionMember1ClauseUnionMember0(BaseModel):
    attribute: str

    operator: Literal[
        "equals",
        "not_equals",
        "greater_than",
        "less_than",
        "greater_than_or_equals",
        "less_than_or_equals",
        "contains",
        "starts_with",
        "ends_with",
        "in",
        "not_in",
    ]

    value: Union[Optional[str], float, bool, Dict[str, object], List[object]]


class RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember0(BaseModel):
    attribute: str

    operator: Literal[
        "equals",
        "not_equals",
        "greater_than",
        "less_than",
        "greater_than_or_equals",
        "less_than_or_equals",
        "contains",
        "starts_with",
        "ends_with",
        "in",
        "not_in",
    ]

    value: Union[Optional[str], float, bool, Dict[str, object], List[object]]


class RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember0(BaseModel):
    attribute: str

    operator: Literal[
        "equals",
        "not_equals",
        "greater_than",
        "less_than",
        "greater_than_or_equals",
        "less_than_or_equals",
        "contains",
        "starts_with",
        "ends_with",
        "in",
        "not_in",
    ]

    value: Union[Optional[str], float, bool, Dict[str, object], List[object]]


class RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember0(BaseModel):
    attribute: str

    operator: Literal[
        "equals",
        "not_equals",
        "greater_than",
        "less_than",
        "greater_than_or_equals",
        "less_than_or_equals",
        "contains",
        "starts_with",
        "ends_with",
        "in",
        "not_in",
    ]

    value: Union[Optional[str], float, bool, Dict[str, object], List[object]]


class RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember0(
    BaseModel
):
    attribute: str

    operator: Literal[
        "equals",
        "not_equals",
        "greater_than",
        "less_than",
        "greater_than_or_equals",
        "less_than_or_equals",
        "contains",
        "starts_with",
        "ends_with",
        "in",
        "not_in",
    ]

    value: Union[Optional[str], float, bool, Dict[str, object], List[object]]


class RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1(
    BaseModel
):
    clauses: List[Union[Optional[str], float, bool, Dict[str, object], List[object]]]

    logical_operator: Literal["AND", "OR"]


RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1Clause: TypeAlias = Union[
    RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember0,
    RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1,
]


class RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1(BaseModel):
    clauses: List[
        RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1Clause
    ]

    logical_operator: Literal["AND", "OR"]


RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1Clause: TypeAlias = Union[
    RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember0,
    RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1,
]


class RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1(BaseModel):
    clauses: List[RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1Clause]

    logical_operator: Literal["AND", "OR"]


RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1Clause: TypeAlias = Union[
    RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember0,
    RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1ClauseUnionMember1,
]


class RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1(BaseModel):
    clauses: List[RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1Clause]

    logical_operator: Literal["AND", "OR"]


RuleConditionUnionMember1ClauseUnionMember1Clause: TypeAlias = Union[
    RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember0,
    RuleConditionUnionMember1ClauseUnionMember1ClauseUnionMember1,
]


class RuleConditionUnionMember1ClauseUnionMember1(BaseModel):
    clauses: List[RuleConditionUnionMember1ClauseUnionMember1Clause]

    logical_operator: Literal["AND", "OR"]


RuleConditionUnionMember1Clause: TypeAlias = Union[
    RuleConditionUnionMember1ClauseUnionMember0, RuleConditionUnionMember1ClauseUnionMember1
]


class RuleConditionUnionMember1(BaseModel):
    clauses: List[RuleConditionUnionMember1Clause]

    logical_operator: Literal["AND", "OR"]


RuleCondition: TypeAlias = Union[RuleConditionUnionMember0, RuleConditionUnionMember1]


class RuleRollout(BaseModel):
    percentage: float
    """Percentage of matching traffic (0–100) served this variation.

    For multi-way splits, use cumulative upper bounds across rules (e.g. 30, 70,
    100).
    """

    attribute: Optional[str] = None
    """Context attribute used for sticky bucketing.

    Defaults to `targetingKey`. If absent at evaluation time, bucketing is random
    per request.
    """


class Rule(BaseModel):
    conditions: List[RuleCondition]
    """Conditions the context must satisfy for this rule to match.

    An empty array matches all contexts.
    """

    priority: int
    """Evaluation order: the API evaluates rules with lower numbers first.

    Must be unique across the flag's rules.
    """

    serve_variation: str
    """Variation the API serves when this rule matches. Must be a key in `variations`."""

    rollout: Optional[RuleRollout] = None


class FlagListResponse(BaseModel):
    default_variation: str
    """
    Variation the API serves when the flag is off, or when it's on but no rule
    matches the context. Must be a key in `variations`.
    """

    enabled: bool
    """When false, the flag bypasses all rules and always serves `default_variation`."""

    key: str
    """Unique identifier for the flag within an app.

    Used in all evaluation and SDK calls.
    """

    rules: List[Rule]
    """Targeting rules evaluated in ascending `priority`; the first matching rule wins.

    An empty array means the flag always serves `default_variation`.
    """

    variations: Dict[str, Union[Optional[str], float, bool, Dict[str, object], List[object]]]
    """Map of variation name to value.

    All values share the same type (boolean, string, number, or JSON object/array),
    and each serialized value stays within 10KB.
    """

    description: Optional[str] = None

    type: Optional[Literal["boolean", "string", "number", "json"]] = None
    """Value type of the flag's variations.

    The API infers this from the variation values on write, so you can omit it in
    requests.
    """

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ......_models import BaseModel

__all__ = [
    "PortalUpdateResponse",
    "Server",
    "ServerAuthConfigSummary",
    "ServerAuthConfigSummaryConfig",
    "ServerAuthConfigSummaryRegistrationInfo",
    "ServerErrorDetails",
    "ServerUpdatedPrompt",
    "ServerUpdatedTool",
]


class ServerAuthConfigSummaryConfig(BaseModel):
    authorization_endpoint: Optional[str] = None

    issuer: Optional[str] = None

    resource: Optional[str] = None

    revocation_endpoint: Optional[str] = None

    token_endpoint: Optional[str] = None


class ServerAuthConfigSummaryRegistrationInfo(BaseModel):
    client_id: Optional[str] = None

    redirect_uris: Optional[List[str]] = None

    scope: Optional[str] = None

    token_endpoint_auth_method: Optional[str] = None


class ServerAuthConfigSummary(BaseModel):
    """Safe subset of auth_credentials surfaced to the dashboard.

    Includes auth_mode (dcr|manual), has_client_secret, client_secret_version, and the OAuth endpoints + client_id for manual servers. Never includes the secret value.
    """

    auth_mode: Optional[Literal["dcr", "manual"]] = None

    client_secret_version: Optional[float] = None

    config: Optional[ServerAuthConfigSummaryConfig] = None

    has_client_secret: Optional[bool] = None

    registration_info: Optional[ServerAuthConfigSummaryRegistrationInfo] = None


class ServerErrorDetails(BaseModel):
    cause: Optional[str] = None
    """Underlying error message"""

    is_upstream: Optional[bool] = None
    """True = MCP server returned an error. False = couldn't reach the server"""

    mcp_code: Optional[float] = None
    """MCP protocol error code"""

    retryable: Optional[bool] = None
    """Whether the error is transient and worth retrying"""

    status_code: Optional[float] = None
    """HTTP status code from the server"""


class ServerUpdatedPrompt(BaseModel):
    name: str

    enabled: Optional[bool] = None

    portal_alias: Optional[str] = None

    portal_description: Optional[str] = None

    server_alias: Optional[str] = None

    server_description: Optional[str] = None


class ServerUpdatedTool(BaseModel):
    name: str

    enabled: Optional[bool] = None

    portal_alias: Optional[str] = None

    portal_description: Optional[str] = None

    server_alias: Optional[str] = None

    server_description: Optional[str] = None


class Server(BaseModel):
    id: str
    """server id"""

    auth_type: Literal["oauth", "bearer", "unauthenticated"]

    hostname: str

    name: str

    prompts: List[Dict[str, object]]

    server_id: str
    """server id"""

    tools: List[Dict[str, object]]

    auth_config_summary: Optional[ServerAuthConfigSummary] = None
    """Safe subset of auth_credentials surfaced to the dashboard.

    Includes auth_mode (dcr|manual), has_client_secret, client_secret_version, and
    the OAuth endpoints + client_id for manual servers. Never includes the secret
    value.
    """

    created_at: Optional[datetime] = None

    created_by: Optional[str] = None

    default_disabled: Optional[bool] = None

    description: Optional[str] = None

    error: Optional[str] = None

    error_details: Optional[ServerErrorDetails] = None

    is_shared_oauth_callback_enabled: Optional[bool] = None
    """
    When true, the gateway worker uses the shared Cloudflare-owned OAuth callback
    endpoint as the redirect_uri for upstream on-behalf OAuth, instead of the
    customer portal hostname. Defaults to false (off); opt in per server by setting
    true. Effective behavior is gated by the gateway worker's per-env rollout mode
    KV key.
    """

    last_successful_sync: Optional[datetime] = None

    last_synced: Optional[datetime] = None

    modified_at: Optional[datetime] = None

    modified_by: Optional[str] = None

    on_behalf: Optional[bool] = None

    secure_web_gateway: Optional[bool] = None
    """Route outbound traffic to this MCP server through Zero Trust Secure Web Gateway"""

    status: Optional[Literal["waiting", "ready", "stale", "error"]] = None
    """Current sync state of the server"""

    updated_prompts: Optional[List[ServerUpdatedPrompt]] = None

    updated_tools: Optional[List[ServerUpdatedTool]] = None


class PortalUpdateResponse(BaseModel):
    id: str
    """portal id"""

    hostname: str

    name: str

    servers: List[Server]

    allow_code_mode: Optional[bool] = None
    """Allow remote code execution in Dynamic Workers (beta)"""

    created_at: Optional[datetime] = None

    created_by: Optional[str] = None

    description: Optional[str] = None

    modified_at: Optional[datetime] = None

    modified_by: Optional[str] = None

    secure_web_gateway: Optional[bool] = None
    """Route outbound MCP traffic through Zero Trust Secure Web Gateway"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)
from .available import (
    AvailableResource,
    AsyncAvailableResource,
    AvailableResourceWithRawResponse,
    AsyncAvailableResourceWithRawResponse,
    AvailableResourceWithStreamingResponse,
    AsyncAvailableResourceWithStreamingResponse,
)

__all__ = [
    "AvailableResource",
    "AsyncAvailableResource",
    "AvailableResourceWithRawResponse",
    "AsyncAvailableResourceWithRawResponse",
    "AvailableResourceWithStreamingResponse",
    "AsyncAvailableResourceWithStreamingResponse",
    "DatasetsResource",
    "AsyncDatasetsResource",
    "DatasetsResourceWithRawResponse",
    "AsyncDatasetsResourceWithRawResponse",
    "DatasetsResourceWithStreamingResponse",
    "AsyncDatasetsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["FindingTypeListResponse", "Category"]


class Category(BaseModel):
    """Category information for a finding."""

    observation: Literal["Issue", "Insight", "Activity"]
    """The type of the observation."""

    product: Literal["SaaS", "Cloud"]
    """The product category."""

    type: Literal["Content", "Posture"]
    """The type of the finding category."""


class FindingTypeListResponse(BaseModel):
    """Basic finding type information."""

    id: str
    """The unique identifier of the finding."""

    category: Category
    """Category information for a finding."""

    name: str
    """The name of the finding."""

    severity: Literal["Critical", "High", "Medium", "Low"]
    """The severity level of a finding."""

    vendor: str
    """The SaaS/Cloud vendor of the platform with which the finding is associated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ......_models import BaseModel

__all__ = ["OrderGetResponse"]


class OrderGetResponse(BaseModel):
    """
    The ordered list of level IDs for a sensitivity group.
    Used to get and set the ordering of levels independently of level attributes.
    """

    level_ids: List[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["RelationshipListParams"]


class RelationshipListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    dataset_id: Required[Annotated[str, PropertyInfo(alias="datasetId")]]
    """The dataset ID to search within."""

    direction: Literal["ancestors", "descendants", "both"]
    """The direction to traverse the graph. Defaults to 'both' to search all."""

    include_parent: Annotated[bool, PropertyInfo(alias="includeParent")]
    """Whether to include the starting event in the results. Defaults to true."""

    indicator_type_ids: Annotated[SequenceNotStr[str], PropertyInfo(alias="indicatorTypeIds")]
    """An optional array of indicator type IDs to filter the results by."""

    max_depth: Annotated[float, PropertyInfo(alias="maxDepth")]
    """The maximum depth to traverse. Defaults to 5."""

    page: float

    page_size: Annotated[float, PropertyInfo(alias="pageSize")]

    relationship_types: Annotated[Union[str, SequenceNotStr[str]], PropertyInfo(alias="relationshipTypes")]
    """An optional array of relationship types to filter by."""
